static char rcsid[] = "@(#)$Id: init.c,v 2.2 2009-10-21 04:22:17 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.2 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI> 
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/


#include "def_misc.h"
#include "s_elm.h"
#include "conf_writer_imp.h"


#if ANSI_C
#define S_(x) static x;
#else
#define S_(x)
#endif

S_(dump_conf_map_f dump_user_iso2022_sets)
static void dump_user_iso2022_sets P_((FILE *f));
static void dump_user_iso2022_sets(f) 
     FILE *f;
{
    dump_iso2022_map(f,ml_user);
}

S_(dump_message_f user_iso2022_sets_written)
static void user_iso2022_sets_written P_((char *fname));
static void user_iso2022_sets_written(fname)
     char *fname;
{
    lib_error(CATGETS(elm_msg_cat, ElmSet, 
		      ElmISO2022SavedIn,
		      "ISO 2022 mappings saved in file %s."), 
	      fname);		    
}

S_(dump_conf_map_f dump_user_mime_charsets)
static void dump_user_mime_charsets P_((FILE *f));
static void dump_user_mime_charsets(f) 
     FILE *f;
{
    dump_charset_map(f,user_charset_map);
}

S_(dump_message_f user_mime_charsets_written)
static void user_mime_charsets_written P_((char *fname));
static void user_mime_charsets_written(fname)
     char *fname;
{
    lib_error(CATGETS(elm_msg_cat, ElmSet, 
		      ElmCharsetsSavedIn,
		      "Charsets saved in file %s."), 
	      fname);
}

S_(dump_conf_map_f dump_user_terminal_info)
static void dump_user_terminal_info P_((FILE *f));
static void dump_user_terminal_info(f) 
     FILE *f;
{
    dump_terminal_map(f,user_terminal_map);
}

S_(dump_message_f user_terminal_info_written)
static void user_terminal_info_written P_((char *fname));
static void user_terminal_info_written(fname)
     char *fname;
{
    lib_error(CATGETS(elm_msg_cat, ElmSet, 
		      ElmTerminalInfoSavedIn,
		      "Additional terminal info saved in file %s."), 
	      fname);
}

S_(dump_conf_map_f dump_user_mime_types)
static void dump_user_mime_types P_((FILE *f));
static void dump_user_mime_types(f) 
     FILE *f;
{
    dump_mime_types_map(f,user_mimetypes_map);
}

S_(dump_message_f user_mime_types_written)
static void user_mime_types_written P_((char *fname));
static void user_mime_types_written(fname)
     char *fname;
{
    lib_error(CATGETS(elm_msg_cat, ElmSet, 
		      ElmContentTypeInfoSavedIn,
		      "Content-type mapping info saved in file %s."), 
	      fname);
}

S_(dump_conf_map_f dump_user_mlist_file)
static void dump_user_mlist_file P_((FILE *f));
static void dump_user_mlist_file(f) 
     FILE *f;
{
    dump_mlist_conf(f,user_mlist_conf);
}

S_(dump_message_f user_mlist_file_written)
static void user_mlist_file_written P_((char *fname));
static void user_mlist_file_written(fname)
     char *fname;
{
    lib_error(CATGETS(elm_msg_cat, ElmSet, 
		      ElmMailingListInfoSavedIn,
		      "Mailing lists info saved in file %s."), 
	      fname);
}



struct terminal_map_item * system_terminal_map  = NULL;
struct terminal_map_item * user_terminal_map    = NULL;
struct mime_types_item   * system_mimetypes_map = NULL;
struct mime_types_item   * user_mimetypes_map   = NULL;
struct mlist_conf        * system_mlist_conf    = NULL;
struct mlist_conf        * user_mlist_conf      = NULL;


#if ANSI_C
static init_default_hook init_misc_defaults;
#endif
static void init_misc_defaults P_((int *errors));
static void init_misc_defaults(errors)
     int *errors;
{
    system_terminal_map  = load_terminal_map(system_terminal_info,errors);
    system_mimetypes_map = load_mime_types_map(system_mime_types,errors);
    system_mlist_conf    = load_mlist_conf(system_mlist_file,errors);


    register_conf_write(USER_ISO2022_SETS,user_iso2022_sets,
			dump_user_iso2022_sets,
			user_iso2022_sets_written);
    register_conf_write(USER_MIME_CHARSETS,user_mime_charsets,
			dump_user_mime_charsets,
			user_mime_charsets_written);
    register_conf_write(USER_TERMINAL_INFO,user_terminal_info,
			dump_user_terminal_info,
			user_terminal_info_written);   
    register_conf_write(USER_MIME_TYPES,user_mime_types,
			dump_user_mime_types,
			user_mime_types_written);
    register_conf_write(USER_MLIST_FILE,user_mlist_file,
			dump_user_mlist_file,
			user_mlist_file_written);
}

#if ANSI_C
static init_default_hook misc_read_rc_file;
#endif
static void misc_read_rc_file P_((int *errors));
static void misc_read_rc_file(errors)
     int *errors;
{
    user_terminal_map  = load_terminal_map(user_terminal_info,errors);
    user_mimetypes_map = load_mime_types_map(user_mime_types,errors);
    user_mlist_conf    = load_mlist_conf(user_mlist_file,errors);
}

void init_misclib()
{

    add_init_default_hook(&init_misc_defaults);
    add_read_rc_file_hook(&misc_read_rc_file);
}



/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
