static char rcsid[] = "@(#)$Id: aliases_map.c,v 2.3 2009-10-21 04:22:18 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.3 $   $Stat$
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI>
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/

#include "def_aliases.h"
#include "s_aliases.h"

DEBUG_VAR(Debug,__FILE__,"alias");


#define ALIASCLASS_magic	0xF60A

struct aliasclass {
    unsigned short magic;        /* ALIASCLASS_magic */


    enum aliasmode   { system_aliasmap, user_aliasmap, private_aliasmap,
		       system_oldmap,  user_oldmap } mode;

    struct aliases_map *  aliases_map;



    struct aliasview_record     ** found_aliases_map;
    int                            found_aliases_map_len;

    int                            need_update; 
    int                            need_write;

    /* Cheat ... */
    struct string              **  delete_list;
    int                            delete_list_len;

};        


static void reset_aliases_cache P_((struct aliasclass *map));
static void reset_aliases_deletions P_((struct aliasclass *map));

void am_free_aliasclass(map)
     struct aliasclass **map;
{
    if (ALIASCLASS_magic != (*map)->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_free_aliasclass",
	      "bad aliasclass magic",0);


    if ((*map)->aliases_map) {
	free_aliases_map(& ((*map)->aliases_map));	
    }

    reset_aliases_cache(*map);
    reset_aliases_deletions(*map);


    (*map)->magic = 0;       /* Invalidate */
    free(*map);
    *map = NULL;
}

struct aliasclass * am_malloc_aliasclass()
{
    struct aliasclass *ret = safe_malloc(sizeof(*ret));

    /* bzero is defined hdrs/defs.h */
    bzero((void *)ret,sizeof (*ret));

    ret->mode        = private_aliasmap;
    ret->aliases_map = NULL;
    
    ret->found_aliases_map     = NULL;
    ret->found_aliases_map_len = 0;

    ret->need_update           = 0;
    ret->need_write            = 0;

    ret->delete_list           = NULL;
    ret->delete_list_len       = 0;

    ret->magic       = ALIASCLASS_magic;

    return ret;
}

static struct aliases_map * get_aliases_map P_((struct aliasclass *map));
static struct aliases_map * get_aliases_map(map)
     struct aliasclass *map;
{

    switch(map->mode) {
    case system_aliasmap:   

	if (map->aliases_map)
	    panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"get_aliases_map",
		  "Mode is not private",0);
	
	if (!system_aliases_map) {
	    DPRINT(Debug,4,(&Debug,
			    "get_aliases_map: Creating empty system_aliases_map\n"));

	    system_aliases_map =  new_aliases_map();
	}
       
	return system_aliases_map;

    case user_aliasmap:     

	if (map->aliases_map)
	    panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"get_aliases_map",
		  "Mode is not private",0);

	if (!user_aliases_map) {
	    DPRINT(Debug,4,(&Debug,
			    "get_aliases_map: Creating empty user_aliases_map\n"));

	    user_aliases_map =  new_aliases_map();
	}

	return user_aliases_map;

    case private_aliasmap: 
    case system_oldmap:
    case user_oldmap:
	return map->aliases_map;
    }

    panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"get_aliases_map",
	  "Bad mode",0);

    return NULL;
}

void am_set_standard_aliasmap(map,user)
     struct aliasclass *map;
     int user;
{
   if (ALIASCLASS_magic != map->magic)
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_standard_aliasmap",
	     "bad aliasclass magic",0);

   if (map->aliases_map) {
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_standard_aliasmap",
	     "aliases_map already set",0);
   }
      

   if (user) 
       map->mode = user_aliasmap;
   else
       map->mode = system_aliasmap;
   
}



void am_set_compat_aliasmap(map,am,mode)
     struct aliasclass  *map;
     struct aliases_map *am;
     enum compat_mode    mode;
{
   if (ALIASCLASS_magic != map->magic)
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_compat_aliasmap",
	     "bad aliasclass magic",0);

   if (map->aliases_map) {
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_compat_aliasmap",
	     "aliases_map already set",0);
   }

   map->aliases_map = am;
   switch (mode) {
   case compat_none:    map->mode = private_aliasmap; break;
   case compat_system:  map->mode = system_oldmap;    break;
   case compat_user:    map->mode = user_oldmap;      break;
   default:
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_compat_aliasmap",
	     "bad mode",0);
   }

}

int give_aliases_filename(map,am,fn)
     struct aliasclass *map;
     struct aliases_map **am;
     char **fn;
{

   if (ALIASCLASS_magic != map->magic)
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__," give_aliases_filename",
	     "bad aliasclass magic",0);


    *am = NULL;
    *fn = NULL;

    switch(map->mode) {
    case system_aliasmap:             /* Actually should not work .... */
	*am  =    system_aliases_map;
	*fn  =    system_aliases_file;

	break;
    case user_aliasmap:    
	*am  =    user_aliases_map;
	*fn  =    user_aliases_file;
	break;
    case  private_aliasmap: 
	*am = map->aliases_map;
	*fn = NULL;                    /* Filename is not known */

	DPRINT(Debug,6,(&Debug,
			"give_aliases_filename: Filename of aliases map is not known (private_aliasmap)\n"));
	
	return 0;

    case system_oldmap:
	*am = map->aliases_map;
	*fn = old_system_text_file;
	break;

    case user_oldmap:
	*am = map->aliases_map;
	*fn = old_user_text_file;
	break;

    default:
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"give_aliases_filename",
	      "Bad mode",0);

    }
    
    return 1;
}




static void reset_aliases_cache(map)
     struct aliasclass *map;
{
    if (map->found_aliases_map) {
	int i;
	
	for (i = 0; i < map->found_aliases_map_len; i++)
	    if (map->found_aliases_map[i])
		free_aliasview_record(& (map->found_aliases_map[i]));
	
	free(map->found_aliases_map);
	map->found_aliases_map = NULL;
    }

    map->found_aliases_map_len = 0; 
}


static void update_aliases_cache P_((struct aliasclass *map));
static void update_aliases_cache(map)
     struct aliasclass *map;
{
    struct aliasview_record ** old_aliases_map =
	map->found_aliases_map;
    int                        old_aliases_map_len =
	map->found_aliases_map_len;
    int i;
 
    map->found_aliases_map     = NULL;
    map->found_aliases_map_len = 0;

    if (! old_aliases_map || 
	!old_aliases_map_len )
	return;

    /* Preallocate new cache */

    map->found_aliases_map = 
	safe_malloc(old_aliases_map_len * 
		    sizeof(map->found_aliases_map[0]));

    for (i = 0; i < old_aliases_map_len; i++)
	map->found_aliases_map[i] = NULL;    
    map->found_aliases_map_len = old_aliases_map_len;


    for (i = 0; i < old_aliases_map_len; i++) {
	int cindex;
	struct aliasview_record *S;

	if (!old_aliases_map[i])
	    continue;

	S = 
	    am_lookup_alias(map,
			    old_aliases_map[i]->alias_key,
			    &cindex);

	if (!S) {
	    DPRINT(Debug,9,(&Debug,
			    "update_aliases_cache: \"%S\" lost\n",
			    old_aliases_map[i]->alias_key));
	    continue;
	}

	DPRINT(Debug,14,(&Debug,
			 "update_aliases_cache: \"%S\": %d => %d\n",
			 old_aliases_map[i]->alias_key,
			 i,
			 cindex));

	S->type   = old_aliases_map[i]->type;
	S->status = old_aliases_map[i]->status;

	free_aliasview_record( & old_aliases_map[i] );
    }


    free(old_aliases_map);
    old_aliases_map = NULL;
}

static void reset_aliases_deletions(map)
     struct aliasclass *map;
{
    if (map->delete_list) {
	int i;
	
	for (i = 0; i < map->delete_list_len; i++) {
	    if (map->delete_list[i])
		free_string(& ( map->delete_list[i] ));
	}

	free(map->delete_list);
	map->delete_list = NULL;
    }
    map->delete_list_len = 0;

}

void am_delete_alias(map,cindex)
     struct aliasclass *map;
     int cindex;
{
    struct aliasview_record * XX;


   if (ALIASCLASS_magic != map->magic)
       panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_delete_alias",
	     "bad aliasclass magic",0);


   XX = am_give_alias(map,cindex);

   if (!XX)
       return;

   /* Just cache deletions */

   map->delete_list = safe_realloc(map->delete_list,
				   (map->delete_list_len+1) * 
				   sizeof(map->delete_list[0]));   

   
   map->delete_list[map->delete_list_len++]
       = dup_string(aliasview_key(XX));  

   /* Mark for writing */
   map->need_write  = 1;

}

struct aliasview_record *am_give_alias(map,cindex)
     struct aliasclass *map;
     int cindex;
{
    struct aliases_map *am = NULL;
    const struct address_alias * alias_value = NULL;
    const struct string        * alias_key   = NULL;

    int type = 0;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_give_alias",
	      "bad aliasclass magic",0);

    if (system_aliasmap == map->mode || 
	system_oldmap   == map->mode)
	type = SYSTEM;

    am = get_aliases_map(map);

    if (!am)
	return NULL;

    if (map->found_aliases_map_len <= cindex) {
	int i;

	map->found_aliases_map = safe_realloc(map->found_aliases_map,
					      (cindex+1) * 
					      sizeof(map->found_aliases_map[0]));
	
	for (i = map->found_aliases_map_len; i <= cindex; i++)
	    map->found_aliases_map[i] = NULL;

	map->found_aliases_map_len = cindex+1;
    }

    alias_value = aliases_map_get_alias(am,cindex,&alias_key);

    DPRINT(Debug,7,(&Debug,"am_give_alias(...,%d): ",cindex));
    if (alias_key) {
	DPRINT(Debug,7,(&Debug,"alias_key=%S ",alias_key));
    }
    DPRINT(Debug,7,(&Debug,"\n"));

    if (!alias_value || !alias_key)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_give_alias",
	      "No alias_value or alias_key",0);

    if (! map->found_aliases_map[cindex])
	map->found_aliases_map[cindex] = 
	    malloc_aliasview_record(alias_key,alias_value,type,0);
    else
	replace_aliasview_record(map->found_aliases_map[cindex],
				 alias_key,alias_value);


    return map->found_aliases_map[cindex];
}

struct aliasview_record *am_lookup_alias(map,key,cindex)
     struct aliasclass *map;
     const struct string *key;
     int *cindex;
{
    struct aliasview_record    * ret         = NULL;

    struct aliases_map *am                   = NULL;
    const struct address_alias * tmp_value   = NULL;
    const struct address_alias * alias_value = NULL;

    int index   = -1;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_lookup_alias",
	      "bad aliasclass magic",0);

    am = get_aliases_map(map);

    tmp_value = aliases_map_lookup_alias(am,key,&index);

    /* am_give_alias is used to instiate correct shared key pointer 
       for alias_key
    */

    if (index < 0) {

	if (tmp_value) {
	    DPRINT(Debug,1,(&Debug,
			    "am_lookup_alias: On value for alias %S is index %d, but alias found %p\n",
			    key,index,tmp_value));
		   
	}
	    
	DPRINT(Debug,9,(&Debug,
			"am_lookup_alias=NULL:  key=%S (index=%d)\n",
			key,index));
	       
	return NULL;
    }

    if (cindex)
	*cindex = index;

    ret = am_give_alias(map,index);

    if (!ret) {
	 DPRINT(Debug,1,(&Debug,
			 "am_lookup_alias: On value for alias %S is index %d, but alias not found by index\n",
			 key,index));
	 return NULL;
    }

    if (ret->alias_value != tmp_value) {
	DPRINT(Debug,1,(&Debug,
			"am_lookup_alias: alias %S, index %d: aliases_map_lookup_alias=%p != %p (found by index)\n",
			key,index, tmp_value,ret->alias_value));

	       }

    return ret;
}

void am_add_alias(map,rec)
     struct aliasclass        *map; 
     struct aliasview_record  *rec;
{
    struct aliases_map                 * am  = NULL;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_add_alias",
	      "bad aliasclass magic",0);
    
    if (ALIASVIEW_record_magic != rec->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_add_alias",
	      "bad record magic",0);

    am = get_aliases_map(map);
    
    if (!am) {
	DPRINT(Debug,1,(&Debug,
			"am_add_alias: \"%S\": No aliases map available\n",
			rec->alias_key));
	return;
    }

    aliases_map_add_alias(am,rec->alias_key,rec->alias_value);
   
    /* Mark for rebuild */
    map->need_update = 1;     
    
    /* Mark for writing */
    map->need_write  = 1;
}

struct aliasview_record * am_set_alias(map,old_index,buffer)
     struct aliasclass        *map;   
     int                       old_index;
     const struct alias_buffer      *buffer;
{
    struct aliasview_record    * ret         = NULL;
    int index = -1;
    struct aliases_map                 * am  = NULL;
    const struct address_alias * tmp_value   = NULL;


    if (ALIASVIEW_buffer_magic != buffer->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_alias",
	      "bad buffer magic",0);
    
    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_alias",
	      "bad aliasclass magic",0);
    
    am = get_aliases_map(map);
    
    if (!am) {
	DPRINT(Debug,1,(&Debug,
			"am_set_alias: \"%S\": No aliases map available\n",
			buffer->alias_key));
	return NULL;
    }

    aliases_map_add_alias(am,buffer->alias_key,buffer->alias_value);

    tmp_value = aliases_map_lookup_alias(am,buffer->alias_key,&index);

    if (!tmp_value) 
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_set_alias",
	      "Added alias not found",0);
	
    if (old_index >= 0 && index != old_index) {
	DPRINT(Debug,1,(&Debug,
			"am_set_alias: \"%S\": Alias position changed, removing old index %d\n",
			buffer->alias_key,old_index));

	/* NOTE: Removing index invalidates all indexes   ... */

	aliases_map_remove_index(am,old_index);

	update_aliases_cache(map);
    } 

  
    /* am_give_alias() and am_lookup_alias() does always lookup so 
       cache not needed to clear otherwise */


    /* Mark for rebuild */
    map->need_update = 1;     

    /* Mark for writing */
    map->need_write  = 1;

    ret = am_give_alias(map,index);

    if (!ret) {
	DPRINT(Debug,1,(&Debug,
			"am_set_alias: On value for alias %S is index %d, but alias not found by index\n",
			buffer->alias_key,index));
	return NULL;
    }

    if (ret->alias_value != tmp_value) {
	DPRINT(Debug,1,(&Debug,
			"am_set_alias: alias %S, index %d: aliases_map_lookup_alias=%p != %p (found by index)\n",
			buffer->alias_key,index, tmp_value,ret->alias_value));
	
    }

    return ret;
}



int am_count_aliases(map)
     struct aliasclass *map;
{
    struct aliases_map *am = NULL;
    int count;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_count_aliases",
	      "bad aliasclass magic",0);

    am = get_aliases_map(map);
    count = aliases_map_item_count(am);

    DPRINT(Debug,7,(&Debug,"am_count_aliases=%d\n",count));

    return count;
}

int am_update_aliases(map)
     struct aliasclass *map;
{
    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__," am_update_aliases",
	      "bad aliasclass magic",0);

    /* FIXME: aliasmap update from file not supported now */

    if (map->need_update) {
	map->need_update = 0;

	update_aliases_cache(map);

	DPRINT(Debug,7,(&Debug,"am_update_aliases=1: Aliases cache update\n"));

	return 1;
    }

    return 0;
}

void am_add_to_alias_stack(stack,map)
     struct alias_stack * stack;
     struct aliasclass *map;
{
    union alias_stack_item map_info;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_add_to_alias_stack",
	      "bad aliasclass magic",0);

    map_info.alias_map = get_aliases_map(map);

    add_lookup_to_alias_stack(stack,alias_simple_lookup,map_info);
}

int am_write_changed(map,rebuild_flag)
     struct aliasclass *map;
     int *rebuild_flag;
{
    char * fn               = NULL;
    struct aliases_map * am = NULL;
    int err;
    char *tmp               = NULL;
    int ret                 = 0;
    FILE *F                 = NULL;
    int i;


    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_write_changed",
	      "bad aliasclass magic",0);

    if (! map->need_write) {
	DPRINT(Debug,9,(&Debug,
			"am_write_changed=1: Write is not needed\n"));
	return 1;
    }  

    if (!give_aliases_filename(map,&am,&fn)) {

	DPRINT(Debug,1,(&Debug,
			"am_write_changed=0: Filename of aliases map is not known\n"));
	return 0;

    }

    if (system_oldmap == map->mode ||
	user_oldmap   == map->mode) {
	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesOldFormatCanNotSaved,
			  "Old format aliases file can not be saved: %s"),
		  fn);
	return 0;
    }

  
    if (!am) {
	DPRINT(Debug,1,(&Debug,
			"am_write_changed: map is not set\n"));
	return 0;
    }


    if (map->delete_list_len) {
	int i;

	DPRINT(Debug,8,(&Debug,
			"am_write_changed: %d deletions\n",
			map->delete_list_len));

	for (i = 0; i < map->delete_list_len; i++) {
	    if (map->delete_list[i]) {
		int r;

		DPRINT(Debug,8,(&Debug,
				"am_write_changed: deleting %S ",
				map->delete_list[i]));

		r = aliases_map_remove_alias(am,map->delete_list[i]);
		
		DPRINT(Debug,8,(&Debug,"%s\n",
				r ? "OK" : "Not found"));		

		if (r)
		    *rebuild_flag = 1;

	    }
	}

	update_aliases_cache(map);

	reset_aliases_deletions(map);
	
	/* Mark for rebuild */
	map->need_update = 1;     
    }

    if (!fn) {
	DPRINT(Debug,1,(&Debug,
			"am_write_changed: filename is not set\n"));
	return 0;
    }

    tmp = elm_message(FRM("%s.N"),fn);
    err = can_open(tmp,"w");

    if (err) {
	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesTmpFileNotWriteable,
			  "Aliases temp file %s is not writeable: %s"),
		  tmp, strerror(err));
	
	goto fail;
    }

    F = fopen(tmp,"w");
    if (!F) {
	int err = errno;

	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesTmpFileNotWriteable,
			  "Aliases temp file %s is not writeable: %s"),
		  tmp, strerror(err));
	
	goto fail;
    }


    dump_aliases_map(F,am);

    if (EOF == fclose(F)) {
	int err = errno;

	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesTmpFileNotWriteable,
			  "Aliases temp file %s is not writeable: %s"),
		  tmp, strerror(err));
	
	goto fail;
    }

    if (0 != rename(tmp,fn)) {
	
	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesTmpFileNotRenamed,
			  "Aliases temp file is not renamed to %s: %s"),
		  fn,strerror(err));

	goto fail;	
    }


    lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesFileWritten,
		      "Aliases file %s is written"),
	      fn);
    ret = 1;
    map->need_write = 0;

    for (i = 0; i < map->found_aliases_map_len; i++) {

	/* Clear NEW flag */

	if (map->found_aliases_map[i])
	    aliasview_clearf_status(map->found_aliases_map[i],NEW);
    }

 fail:
    if (tmp)
	free(tmp);
    
    
    DPRINT(Debug,9,(&Debug,
		    "am_write_changed=%d\n",ret));
    
    return ret;
}

int am_reload_aliases(map)
     struct aliasclass *map;
{
    char * fn               = NULL;
    struct aliases_map * am = NULL;
    int errors = 0;
    struct aliases_map * tmp = NULL;
    int ret = 0;

    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_reload_aliases",
	      "bad aliasclass magic",0);


    if (!give_aliases_filename(map,&am,&fn)) {

	DPRINT(Debug,1,(&Debug,
			"am_write_changed=0: Filename of aliases map is not known\n"));
	return 0;

    }
    
    if (!fn) {
	DPRINT(Debug,1,(&Debug,
			"am_reload_aliases: filename is not set\n"));
	return 0;
    }

    tmp = load_aliases_map(fn,&errors);

    if (!tmp || errors) {
	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesFileNotReloaded,
			  "Failed to reload aliases file: %s"),
		  fn);

	goto failure;
    }

    switch(map->mode) {
    case system_aliasmap:             /* Actually should not work .... */

	if (system_aliases_map)
	    free_aliases_map(& system_aliases_map);

	system_aliases_map = tmp;
	tmp = NULL;
	ret = 1;
	break;

    case user_aliasmap:    
	if (user_aliases_map)
	    free_aliases_map(&  user_aliases_map);
	
	user_aliases_map = tmp;
	tmp = NULL;
	ret = 1;
	break;
    
    case system_oldmap:          /* Actually should not work .... */
    case user_oldmap:
    case private_aliasmap:       /* Should not be reached ... */

	if (map->aliases_map)
	    free_aliases_map(& map->aliases_map);
	
	map->aliases_map = tmp;
	tmp = NULL;
	ret = 1;
	break;
	       
    default:
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_reload_aliases",
	      "Bad mode",0);

    }
 
    if (ret)
	lib_error(CATGETS(elm_msg_cat, AliasesSet, AliasesFileReloaded,
			  "Aliases file %s is reloaded"),
		  fn);

 failure:
    if (tmp)
	free_aliases_map(&tmp);

    DPRINT(Debug,9,(&Debug,
		    "am_reload_aliases=%d fn=%s\n",
		    ret, fn ? fn : "<none>"));

    return ret;
}


const char * am_is_user_editable(map)
     struct aliasclass *map;
{
    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_is_user_editable",
	      "bad aliasclass magic",0);

    if (user_aliasmap == map->mode)
	return user_aliases_file;

    return NULL;
}

int am_is_normal_target(map)
     struct aliasclass *map;
{
    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_is_normal_target",
	      "bad aliasclass magic",0);

    return 
	system_aliasmap == map->mode ||
	user_aliasmap   == map->mode;

}

int am_is_compat_map(map)
     struct aliasclass *map;
{
    if (ALIASCLASS_magic != map->magic)
	panic("ALIAS VIEW PANIC",__FILE__,__LINE__,"am_is_compat_map",
	      "bad aliasclass magic",0);

    return 
	system_oldmap == map->mode ||
	user_oldmap   == map->mode;
}


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
